@@ -1,10 +1,12 @@
-## ----setup, echo=FALSE, results="hide"--------------------------------------------------
+## ----setup, results="hide"--------------------------------------------------------------
 library("knitr")
-opts_chunk$set(fig.align="center", fig.width=7, fig.height=7)
+opts_chunk$set(fig.align="center", fig.width=6, fig.height=6)
 options(width=90)
 
-## ---------------------------------------------------------------------------------------
+## ----data-------------------------------------------------------------------------------
 library("corrgram")
+head(baseball)
+round(cor(baseball[, 5:14], use="pair"),2)
 
 ## ----fig2-------------------------------------------------------------------------------
 vars2 <- c("Assists","Atbat","Errors","Hits","Homer","logSal",
@@ -21,6 +23,7 @@ e1 <- baseball.eig[,1]
 e2 <- baseball.eig[,2]
 plot(e1,e2,col='white', xlim=range(e1,e2), ylim=range(e1,e2))
 text(e1,e2, rownames(baseball.cor), cex=1)
+title("Eigenvector plot of baseball data")
 arrows(0, 0, e1, e2, cex=0.5, col="red", length=0.1)
 
 ## ----fig4-------------------------------------------------------------------------------
@@ -70,7 +73,7 @@ partial <- function(r, xvar){
   s <- diag(ri)
   s <- diag(sqrt(1/s))
   ri <- s %*% ri %*% s
-  ri <- as.matrix(bdiag(ri, r[xvar, xvar]))
+  ri <- as.matrix(Matrix::bdiag(ri, r[xvar, xvar]))
   diag(ri) <- 1  # Should already be 1, but could be 1 + epsilon
   colnames(ri) <- rownames(ri) <- c(yvar, xvar)
   return(ri)
@@ -82,15 +85,35 @@ vars8b <- c("MPG", "Price")
 vars8 <- c(vars8a, vars8b)
 auto.cor <- cor(auto[, vars8], use="pair")
 auto.par <- partial(auto.cor, vars8b)
-corrgram(auto.par, lower.panel=panel.pie, upper.panel=panel.pie,
+corrgram(auto.par,
+         lower.panel=panel.pie, upper.panel=panel.pie,
          main="Auto data, partialing out Price,MPG")
 
-## ---------------------------------------------------------------------------------------
+## ----ellipse----------------------------------------------------------------------------
 corrgram(baseball[,vars2], order=TRUE,
          main="Baseball correlation ellipses",
          panel=panel.ellipse,
          text.panel=panel.txt, diag.panel=panel.minmax)
 
-## ----finish, echo=FALSE, results="asis"-------------------------------------------------
-toLatex(sessionInfo(), locale=FALSE)
+## ----scatter----------------------------------------------------------------------------
+corrgram(iris,
+         main="Iris data with example panel functions",
+         lower.panel=panel.pts, upper.panel=panel.conf,
+         diag.panel=panel.density)
+
+## ----panelbar---------------------------------------------------------------------------
+corrgram(auto, order=TRUE,
+         main="Auto data (PC order)",
+         lower.panel=corrgram::panel.ellipse,
+         upper.panel=panel.bar, diag.panel=panel.minmax,
+         col.regions=colorRampPalette(c("darkgoldenrod4", "burlywood1",
+                                        "darkkhaki", "darkgreen")))
+
+## ----vote-------------------------------------------------------------------------------
+# 'vote' is a correlation matrix, not a data frame
+corrgram(vote, order=TRUE,
+         upper.panel=panel.cor, main="vote")
+
+## ----session----------------------------------------------------------------------------
+sessionInfo()
 
